FUNCTION g(x,y)
USE jm_constants
USE jm_global, ONLY: ntrunc, nend, scheme, l, kappa, lambda, basis, pass, &
                     vn_is_loaded, fout, e, estart

! Computes elements of the matrix approximating Green function.

IMPLICIT none

INTEGER, INTENT(IN)              :: x, y
REAL(KIND=dp)                    :: g
REAL(KIND=dp)                    :: jm, vn
REAL(KIND=dp), ALLOCATABLE       :: a(:,:), eigval(:), eigvec(:,:), v(:,:)
REAL(KIND=dp), ALLOCATABLE, SAVE :: a_saved(:,:), v_saved(:,:)
INTEGER                          :: dim, maxdim, i, j, ii


IF (scheme == 'relativistic') THEN
  dim    = 2 * ntrunc
  maxdim = 2 * nend
ELSE
  dim    = ntrunc
  maxdim = nend
ENDIF

! Allocate work matrices
ALLOCATE(v(0:dim-1,0:dim-1), a(dim,dim), eigval(dim), eigvec(dim,dim))

! Create the matrix <a> = <j> + <v>
IF (.NOT. allocated(a_saved)) THEN    ! The first call.
  IF (pass == 1) THEN
    ! Allocate matrices to store the elements computed for given ntrunc
    ! for use in the next calls of the subroutine.
    ! Matrix <a_saved> can be used only in the first pass with a fixed energy.
    ! Matrix <v_saved> is used in both passes.
    ALLOCATE(v_saved(0:maxdim-1,0:maxdim-1),a_saved(maxdim,maxdim))
  ELSE      ! 2nd pass - restore matrix v from v_saved 
    DO i=0,dim-1
      DO j=0,dim-1
        v(i,j) = v_saved(i,j)
      ENDDO
    ENDDO
  ENDIF

  DO i=1,dim
    DO j=1,i
      IF (pass==1 .OR. (pass==2 .and. e==estart)) THEN     ! First run
        v(i-1,j-1) = vn(i-1,j-1)         
        v(j-1,i-1) = v(i-1,j-1)
      ENDIF
      a(i,j) = jm(i-1,j-1) + v(i-1,j-1)
      a(j,i) = a(i,j)                    ! Utilize symmetry of the matrix
    ENDDO
  ENDDO

ELSE  ! Use the elements stored in the matrices <a_saved> and <v_saved>
  IF (scheme == 'relativistic') THEN
    DO i=2,dim-1
      DO j=2,dim-1
        a(i,j) = a_saved(i-1,j-1)
        v(i-1,j-1) = v_saved(i-2,j-2)
      ENDDO
    ENDDO
    DO i=1,dim
      v(dim-1,i-1) = vn(dim-1,i-1)
      a(dim,i) = jm(dim-1,i-1) + v(dim-1,i-1)
      a(i,dim) = a(dim,i)
      v(i-1,dim-1) = v(dim-1,i-1)
      v(0,i-1) = vn(0,i-1)
      a(1,i)   = jm(0,i-1) + v(0,i-1)
      a(i,1)   = a(1,i)
      v(i-1,0) = v(0,i-1)
    ENDDO
  ELSE
    DO i=1,dim-1
      DO j=1,dim-1
        a(i,j) = a_saved(i,j)
        v(i-1,j-1) = v_saved(i-1,j-1)
      ENDDO
    ENDDO
    DO i=1,dim
      v(dim-1,i-1) = vn(dim-1,i-1)
      a(dim,i) = jm(dim-1,i-1) + v(dim-1,i-1)
      a(i,dim) = a(dim,i)
      v(i-1,dim-1) = v(dim-1,i-1)
    ENDDO
  ENDIF
ENDIF


IF (pass == 1) THEN
  IF (ntrunc == nend) THEN
    DEALLOCATE(a_saved)
    IF (.NOT. vn_is_loaded) THEN
      OPEN(5,file='saved.vn')
      CALL jm_header(5)
      WRITE(5,'(3I3,F5.2,A9,A1,A16)') nend,l,kappa,lambda,basis,' ',scheme
      DO i=1,dim
        DO j=1,dim
           WRITE(5,*) v(i-1,j-1)         ! Write elements of the truncated
        ENDDO                            ! potential to file to future use.
      ENDDO
      CLOSE(5)
    ENDIF
  ELSE 
    DO i=1,dim
      DO j=1,dim
        a_saved(i,j) = a(i,j)
        v_saved(i-1,j-1) = v(i-1,j-1)
      ENDDO
    ENDDO
  ENDIF
ELSE  
  IF (e == estart) THEN       ! First run of the 2nd pass, save <v>
    DO i=0,dim-1
      DO j=0,dim-1
        v_saved(i,j) = v(i,j)
      ENDDO
    ENDDO
  ENDIF
ENDIF


! Finds the eigenvalues and eigenvectors of the matrix.
CALL eigensys(dim,a,eigval,eigvec)

DEALLOCATE(a,v)

! Looks for the matrix element <g>,
! used in the formula for tangent to the phase shift.
IF (scheme == 'relativistic') THEN
  i  = 1
  ii = 1
ELSE
  i  = x+1
  ii = y+1
ENDIF

! Form the matrix g
g = zero
DO j=1,dim
  g = g + eigvec(i,j)*eigvec(ii,j)/eigval(j)
ENDDO

DEALLOCATE(eigval, eigvec)

RETURN

END FUNCTION g

!***********************************************************************
