MODULE jm_global
USE jm_constants

! This module contains definition of the global variables used in program
! and three subroutines which read data from the file and control the
! process of computing the numerical values of the phase shifts.

CHARACTER (LEN=8),  SAVE      :: basis, vol, pot_type
CHARACTER (LEN=16), SAVE      :: scheme
CHARACTER (LEN=99), SAVE      :: fin, fout
REAL(KIND=dp),      SAVE      :: e, k, kpr, lambda
REAL(KIND=dp),      SAVE      :: estart, eend, estep
REAL(KIND=dp),      SAVE      :: v0, a, b
REAL(KIND=dp),      SAVE      :: r0, z, alpha
REAL(KIND=dp),      SAVE      :: eps, epspr, v_light
INTEGER,            SAVE      :: l, kappa, nstart, nend, ntrunc, pass
LOGICAL,            SAVE      :: scr, shift, vn_is_loaded=.FALSE.
INTEGER,            SAVE      :: par(20,2)

CONTAINS


SUBROUTINE pass1()
USE jm_constants

! The main loop of the program - calculate phase shifts for
! <ntrunc> = <nstart>... <nend> with a fixed energy <estart>

IMPLICIT none

REAL(KIND=dp) :: t
REAL(KIND=dp) :: delta, tan_delta

10 FORMAT (A3,I3,A9,F20.16)
20 FORMAT (I3,F22.16,F22.16)

pass = 1

CALL jm_header(2)

! Compute the analytical value if possible
IF (pot_type=='well' .AND. scheme=='relativistic') CALL t_analytic()

IF (scr) WRITE(*,*)
WRITE(2,'(A44)') '; NUMERICAL RESULTS:  N   tan(delta)   delta'
IF (scr) WRITE(*,'(A)')  &
 'Calculating numerical phase shifts for a fixed energy:'



DO ntrunc = nstart, nend
  
  tan_delta = t(ntrunc)            ! Compute the tan(delta)
  delta = atan(tan_delta)          ! and delta for given ntrunc

  IF (shift .AND. delta < zero) delta = delta + pi

  ! Write results to file and (optionally) to screen  
  IF (scr) WRITE(*,10) 'n=',ntrunc,'delta=',delta
  WRITE(2,20) ntrunc, tan_delta, delta
 
ENDDO

RETURN

END SUBROUTINE pass1

!*********************************************************************

SUBROUTINE pass2()
USE jm_constants

! Calculate phase shifts for energies from range [<estart>, <eend>] 
! for <ntrunc> = <nend>

IMPLICIT none

REAL(KIND=dp) :: t
REAL(KIND=dp) :: delta, tan_delta
INTEGER       :: i, iend

30 FORMAT (A8,F10.6,A8,F20.16)
40 FORMAT (F10.6,F22.16,F22.16)

pass = 2
CALL jm_header(3)

IF (scr) WRITE(*,*)
IF (scr) WRITE(*,'(A58,I4,A1)') &
     'Now calculating phase shifts for a given energy range; n =',nend,":"

WRITE(3,'(A46)') '; NUMERICAL RESULTS: energy  tan(delta)  delta'


iend =  int( (eend - estart) / estep + 1.1 )

DO i=1, iend

  e = estart + estep * (i-1)

  IF (scheme == 'relativistic') THEN
    k     = sqrt( e * (e + two * v_light**2 ) ) / v_light
    eps   = sqrt( e / (e + two * v_light**2) )
  ELSE
    k   = sqrt( two * e )
  ENDIF

  ntrunc = nend

  tan_delta = t(ntrunc)            ! Compute the tan(delta)
  delta = atan(tan_delta)          ! and delta for ntrunc = nend

  IF (shift .AND. delta < zero) delta = delta + pi

  ! Write results to file and (optionally) to screen.
  IF (scr) WRITE(*,30) 'energy=',e,'delta=',delta
  WRITE(3,40) e, tan_delta, delta
  
ENDDO

RETURN
END SUBROUTINE pass2

!***********************************************************************

SUBROUTINE loadparams(fin)
USE jm_constants

! Reads data from input file <fin>.

IMPLICIT none

CHARACTER (LEN=*), INTENT(in) :: fin
CHARACTER (LEN=44)            :: item, eqsign, value_char
CHARACTER (LEN=80)            :: line
CHARACTER (LEN=10)            :: par_txt(20)
INTEGER                       :: par(20,2)
INTEGER                       :: i, j


OPEN(1,file=fin,err=1,status='old')

! <par_txt> - parameter labels 
! <par(i)> = 1 if i-th parameter is required
par = 0
par_txt(1)  = 'estart'   ; par(1,1)  = 1 ! required
par_txt(2)  = 'eend'     ; par(2,1)  = 1 ! required
par_txt(3)  = 'estep'    ; par(3,1)  = 0 ! default = (<eend>-<estart>)/10
par_txt(4)  = 'l'        ; par(4,1)  = 1 ! required
par_txt(5)  = 'kappa'    ; par(5,1)  = 0 ! required if <scheme>='relativistic'
par_txt(6)  = 'lambda'   ; par(6,1)  = 0 ! default = 1.0
par_txt(7)  = 'basis'    ; par(7,1)  = 1 ! required
par_txt(8)  = 'scheme'   ; par(8,1)  = 0 ! default = 'non-relativistic'
par_txt(9)  = 'v_light'  ; par(9,1)  = 0 ! default = 'finite'
par_txt(10) = 'pot_type' ; par(10,1) = 0 ! default = 'well'
par_txt(11) = 'screen'   ; par(11,1) = 0 ! default = .true.
par_txt(12) = 'shift'    ; par(12,1) = 0 ! default = .false.
par_txt(13) = 'nstart'   ; par(13,1) = 0 ! default = 1
par_txt(14) = 'nend'     ; par(14,1) = 1 ! required
par_txt(15) = 'v0'       ; par(15,1) = 0 ! required if <pot_type>='well'
par_txt(16) = 'a'        ; par(16,1) = 0 ! required if <pot_type>='well'
par_txt(17) = 'b'        ; par(17,1) = 0 ! required if <pot_type>='well'
par_txt(18) = 'r0'       ; par(18,1) = 0 ! required if <pot_type>='coulomb'
par_txt(19) = 'z'        ; par(19,1) = 0 ! required if <pot_type>='coulomb'
par_txt(20) = 'alpha'    ; par(20,1) = 0 ! required if <pot_type>='coulomb'

WRITE(*,'(A)') 'Loading parameters...'

DO
  READ (1,'(A80)',end=555) line
  IF (index(line,';')==1) THEN    ! line is a comment - read next
    CYCLE
  ELSE
    BACKSPACE(1)
  ENDIF

  READ (1,*,end=555) item, eqsign, value_char
  IF (index(eqsign,'=') /= 1) CYCLE   ! No '=' sign in line - read next


  SELECT CASE (item)

  CASE ('estart')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, estart
    par(1,2) = 1
    PRINT '(A14,F8.4)', ' '//par_txt(1)//' = ', estart

  CASE ('eend')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, eend
    par(2,2) = 1  
    PRINT '(A14,F8.4)', ' '//par_txt(2)//' = ', eend

  CASE ('estep')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, estep
    par(3,2) = 1
    PRINT '(A14,F8.4)', ' '//par_txt(3)//' = ', estep

  CASE ('l')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, l  
    par(4,2) = 1
    PRINT '(A14,I5)', ' '//par_txt(4)//' = ', l

  CASE ('kappa')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, kappa
    par(5,2) = 1
    PRINT '(A14,I5)', ' '//par_txt(5)//' = ', kappa

  CASE ('lambda')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, lambda
    par(6,2) = 1
    PRINT '(A14,F8.4)', ' '//par_txt(6)//' = ', lambda

  CASE ('basis')
    basis = trim(value_char)
    par(7,2) = 1
    PRINT *, par_txt(7),' = ', basis
  
  CASE ('scheme')
    scheme = trim(value_char)
    IF (scheme /= 'relativistic') scheme= 'non-relativistic'
    par(8,2) = 1
    PRINT *, par_txt(8),' = ', scheme

  CASE ('v_light')
    vol = trim(value_char)
    par(9,2) = 1  
    PRINT *, par_txt(9),' = ', vol

  CASE ('pot_type')
    pot_type = trim(value_char)
    par(10,2) = 1
    PRINT *, par_txt(10),' = ', pot_type

  CASE ('screen')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, scr
    par(11,2) = 1
    PRINT *, par_txt(11),' = ', scr

  CASE ('shift')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, shift
    par(12,2) = 1
    PRINT *, par_txt(12),' = ', shift

  CASE ('nstart')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, nstart
    par(13,2) = 1
    PRINT '(A14,I5)', ' '//par_txt(13)//' = ', nstart

  CASE ('nend')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, nend
    par(14,2) = 1
    PRINT '(A14,I5)', ' '//par_txt(14)//' = ', nend

  CASE ('v0')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, v0
    par(15,2) = 1
    PRINT '(A14,F8.4)', ' '//par_txt(15)//' = ', v0

  CASE ('a')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, a
    par(16,2) = 1  
    PRINT '(A14,F8.4)', ' '//par_txt(16)//' = ', a

  CASE ('b')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, b
    par(17,2) = 1  
    PRINT '(A14,F8.4)', ' '//par_txt(17)//' = ', b

  CASE ('r0')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, r0
    par(18,2) = 1  
    PRINT '(A14,F8.4)', ' '//par_txt(18)//' = ', r0

  CASE ('z')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, z
    par(19,2) = 1
    PRINT '(A14,F8.4)', ' '//par_txt(19)//' = ', z

  CASE ('alpha')
    BACKSPACE(1)
    READ(1,*,err=111) item, eqsign, alpha
    par(20,2) = 1  
    PRINT '(A14,F8.4)', ' '//par_txt(20)//' = ', alpha
  
  CASE DEFAULT
 
    PRINT *, '(loadparams) Warning: unknown parameter <', trim(item), &
    '> with value <', trim(value_char), '> has been ignored.'
  
  END SELECT

ENDDO

555 CLOSE(1)

! Complete the requirements table <par>
IF (scheme == 'relativistic') par(5,1) = 1 
IF (pot_type == 'well') THEN
  par(15,1) = 1
  par(16,1) = 1  
  par(17,1) = 1 
ENDIF
IF (pot_type == 'coulomb') THEN
  par(18,1) = 1
  par(19,1) = 1  
  par(20,1) = 1 
ENDIF

! Determine if all required parameters have been read
DO i=1,20
  IF (par(i,1)==1 .AND. par(i,2) == 0) THEN 
    PRINT*, '(loadparams) Error: required parameter missing - ',  &
             trim(par_txt(i)),'.'
    STOP
  ENDIF
ENDDO


! Default values for non-read parameters
DO i=1,20
  IF (par(i,1)==0 .AND. par(i,2) == 0) THEN 
    SELECT CASE(i)
    CASE(3)
      estep = ABS(eend - estart) / 10.0_dp
      PRINT*, '(default) estep   = ', estep
    CASE(6)
      lambda = one
      PRINT*, '(default) lambda  = ', lambda
    CASE(8)
      scheme = 'non-relativistic'
      PRINT*, '(default) scheme  = ',scheme
    CASE(9)
      IF (scheme == 'relativistic') THEN
        vol = 'finite'
        PRINT*, '(default) v_light = ',vol
      ENDIF
    CASE(11)
      scr = .true.
      PRINT*, '(default) screen  = ', scr
    CASE(12)
      shift = .false.
      PRINT*, '(default) shift   = ', shift
    CASE(13)
      nstart = 1
      PRINT '(A20,I5)', ' (default) nstart  = ', nstart
    END SELECT
  ENDIF
ENDDO

e = estart

IF (v0 > e) STOP '(loadparams) Error: v0 > E.'

IF (nstart < 0 .OR. nstart > nend) &
     STOP '(setparams) Error: incorrect values for <nstart> or <nend>.'

IF (kappa /= l .AND. kappa /= -l-1 .AND. scheme == 'relativistic')  &
   STOP '(setparams) Error: the quantum numbers are not properly correlated.'

IF (a >= b) STOP '(setparams) Error: incorrect values for <a> or <b>'

IF (vol == 'finite') THEN
  v_light = 137.036_dp
ELSE
  v_light = 5000.0_dp
ENDIF

IF (scheme == 'relativistic') THEN
  k     = sqrt( e * (e + two * v_light**2 ) ) / v_light
  kpr   = sqrt( (e-v0) * ((e-v0) + two * v_light**2 ) ) / v_light
  eps   = sqrt( e / (e + two * v_light**2) )
  epspr = sqrt( (e-v0) / ((e-v0) + two * v_light**2) )
ELSE
  k   = sqrt( two * e )
  kpr = sqrt( two * (e - v0 ))
ENDIF

RETURN

111 PRINT *, '(loadparams) Error: incorrect value for <', trim(item), '>.'
STOP

1 STOP '(loadparams) rror: cannot open the input file.'
END SUBROUTINE loadparams


END MODULE jm_global
