FUNCTION bessel(v,z)
USE jm_constants

! Bessel function of the first kind.

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: v, z
REAL(KIND=dp)             :: res(100), v_frac
REAL(KIND=dp)             :: hyperg01, gamma
INTEGER                   :: nerr, v_int
REAL(KIND=dp)             :: bessel

v_int  = INT(v)
v_frac = v - v_int

IF (v < zero) THEN
  
  IF (v == INT(v)) THEN

    CALL rjbesl(z, -v_frac, -v_int+1, res, nerr)
    bessel = (-1)**v * res(-v_int+1)
    
  ELSE  ! Direct formula (may be wrong for huge <z> and <v>)

    bessel = (z/two)**v / gamma(v+one) * hyperg01(v+one,-0.25_dp * z**2)
    
  ENDIF
  
ELSE    ! Positive <v> values

  CALL rjbesl(z, v_frac, v_int+1, res, nerr)
  bessel = res(v_int+1)
  
ENDIF

IF (nerr < 0) STOP '(bessel) Unknown error.'

RETURN
END FUNCTION bessel

!***********************************************************************

FUNCTION bess_prim(v,z)
USE jm_constants

! Derivative to the Bessel function.

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: v, z
REAL(KIND=dp)             :: bess_prim
REAL(KIND=dp)             :: bessel

bess_prim = half * ( bessel(v-one,z) - bessel(v+one,z) )

RETURN
END FUNCTION bess_prim

!***********************************************************************

FUNCTION sphbess(v,z)
USE jm_constants

! Spherical Bessel function.

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: v, z
REAL(KIND=dp)             :: sphbess
REAL(KIND=dp)             :: bessel

IF (z == zero) STOP '(sphbess) Error: argument can not be zero.'

sphbess = sqrt(pi/two/z) * bessel(v+half,z)

RETURN
END FUNCTION sphbess

!***********************************************************************

FUNCTION sphbess_prim(v,z)
USE jm_constants

! Derivative to the spherical Bessel function.

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: v, z
REAL(KIND=dp)             :: sphbess_prim
REAL(KIND=dp)             :: sphbess

sphbess_prim = (v*sphbess(v-1,z) - (v+1)*sphbess(v+1,z)) / (2*v+1)

END FUNCTION sphbess_prim

!***********************************************************************

FUNCTION neumann(v,z)
USE jm_constants

! Neumann function (Bessel function of the second kind).

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: v, z
REAL(KIND=dp)             :: neumann
REAL(KIND=dp)             :: res(100)
REAL(KIND=dp)             :: v_frac
REAL(KIND=dp)             :: bessel
INTEGER                   :: v_int, nerr

v_int = INT(v)
v_frac = v - v_int

IF (v < zero) THEN
  
  IF (v == INT(v)) THEN  ! Negative integer vaules

    CALL rybesl(z, -v_frac, -v_int+1, res, nerr)
    neumann = (-1)**v * res(-v_int+1)
    
  ELSE    ! Direct formula

    neumann = one / sin(v*pi) * ( bessel(v,z)*cos(v*pi)-bessel(-v,z)   )
    
  ENDIF
  
ELSE

  CALL rybesl(z, v_frac, v_int+1, res, nerr)
  neumann = res(v_int+1)
  
ENDIF

IF (nerr < 0) STOP '(neumann) Unknown error.'

RETURN
END FUNCTION neumann

!***********************************************************************

FUNCTION neum_prim(v,z)
USE jm_constants

! Derivative to the Neumann function.

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: v, z
REAL(KIND=dp)             :: neum_prim
REAL(KIND=dp)             :: neumann

neum_prim = half * ( neumann(v-one,z) - neumann(v+one,z) )

RETURN
END FUNCTION neum_prim

!***********************************************************************

FUNCTION sphneum(v,z)
USE jm_constants

! Spherical Neumann function

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: v, z
REAL(KIND=dp)             :: sphneum
REAL(KIND=dp)             :: neumann

IF (z == zero) STOP '(sphneum) Error: the argument can not be zero.'

sphneum = sqrt(pi/two/z) * neumann(v+half,z)

RETURN
END FUNCTION sphneum

!***********************************************************************

FUNCTION sphneum_prim(v,z)
USE jm_constants

! Derivative to the spherical Neumann function.

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: v, z
REAL(KIND=dp)             :: sphneum_prim
REAL(KIND=dp)             :: sphneum

sphneum_prim = (v*sphneum(v-1,z) - (v+1)*sphneum(v+1,z)) / (2*v+1)

END FUNCTION sphneum_prim

!***********************************************************************

FUNCTION k_delta(a,b)
USE jm_constants

! The Kronecker delta

IMPLICIT none

INTEGER, INTENT(IN) :: a, b
REAL(KIND=dp)       :: k_delta

IF ( a == b ) THEN
  k_delta = one
ELSE
  k_delta = zero
ENDIF

RETURN
END FUNCTION k_delta

!***********************************************************************

FUNCTION gamma(x)
USE jm_constants

! The gamma function

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: x
REAL(KIND=dp)             :: gamma
REAL(KIND=dp)             :: dgamma

gamma = dgamma(x)

IF (gamma == 1.d308) STOP '(gamma) Error: improper argument.'

RETURN
END FUNCTION gamma

!***********************************************************************

FUNCTION lgamma(x)
USE jm_constants

! Logarithm to the gamma function.

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: x
REAL(KIND=dp)             :: lgamma
REAL(KIND=dp)             :: dlgamma

lgamma = dlgamma(x)

RETURN
END FUNCTION lgamma

!***********************************************************************

FUNCTION gegen(n,l,x)
USE jm_constants

! The Gegenbauer polynomials for argument -1 < <x> < 1 for the integer <l>.

IMPLICIT none

INTEGER,       INTENT(IN) :: n, l
REAL(KIND=dp), INTENT(IN) :: x
REAL(KIND=dp)             :: gegen
REAL(KIND=dp)             :: gamma, lfact
INTEGER                   :: m

IF (abs(x) > one) STOP '(gegen) Error: improper argument.'

gegen = zero

DO m=0,n
  gegen = gegen + exp(lfact(l+m-1)-lfact(m)) * cos((dble(n-2*m))*acos(x)) &
      * exp(lfact(l+n-m-1)-lfact(n-m)) / gamma(dble(l))**2
ENDDO

RETURN
END FUNCTION gegen

!***********************************************************************

FUNCTION laguerre(n,a,x)
USE jm_constants

! Associated Laguerre polynomials.

IMPLICIT none

INTEGER,       INTENT(IN) :: n
REAL(KIND=dp), INTENT(IN) :: a, x
REAL(KIND=dp)             :: laguerre
REAL(KIND=dp)             :: gamma, hyperg11, lgamma, lfact

laguerre = exp(lgamma(n+a+1)-lfact(n))/gamma(a+1)*hyperg11(-dble(n),a+1,x)

RETURN
END FUNCTION laguerre

!***********************************************************************

FUNCTION fact(n)
USE jm_constants

! Factorial of the natural number <n>. The output is real-type.
! Values are tabulated to speed-up calculations

IMPLICIT none

INTEGER, INTENT(IN) :: n
REAL(KIND=dp)       :: fact
REAL(KIND=dp), SAVE :: t_f(0:170)


IF (n < 0)   STOP '(fact) Error: negative argument.'
IF (n > 170) STOP '(fact) Error: argument too big.'

IF (t_f(0) /= one) THEN                  ! First call - tabulate values
  t_f(  0)=   0.1000000000000000D+01
  t_f(  1)=   0.1000000000000000D+01
  t_f(  2)=   0.2000000000000000D+01
  t_f(  3)=   0.6000000000000000D+01
  t_f(  4)=   0.2400000000000000D+02
  t_f(  5)=   0.1200000000000000D+03
  t_f(  6)=   0.7200000000000000D+03
  t_f(  7)=   0.5040000000000000D+04
  t_f(  8)=   0.4032000000000000D+05
  t_f(  9)=   0.3628800000000000D+06
  t_f( 10)=   0.3628800000000000D+07
  t_f( 11)=   0.3991680000000000D+08
  t_f( 12)=   0.4790016000000000D+09
  t_f( 13)=   0.6227020800000000D+10
  t_f( 14)=   0.8717828915200000D+11
  t_f( 15)=   0.1307674411008000D+13
  t_f( 16)=   0.2092279057612800D+14
  t_f( 17)=   0.3556874146283520D+15
  t_f( 18)=   0.6402373705728000D+16
  t_f( 19)=   0.1216451004088320D+18
  t_f( 20)=   0.2432902008176640D+19
  t_f( 21)=   0.5109094217170944D+20
  t_f( 22)=   0.1124000727777608D+22
  t_f( 23)=   0.2585201673888498D+23
  t_f( 24)=   0.6204484017332394D+24
  t_f( 25)=   0.1551121004333099D+26
  t_f( 26)=   0.4032914611266057D+27
  t_f( 27)=   0.1088886945041835D+29
  t_f( 28)=   0.3048883446117138D+30
  t_f( 29)=   0.8841761993739701D+31
  t_f( 30)=   0.2652528598121910D+33
  t_f( 31)=   0.8222838654177922D+34
  t_f( 32)=   0.2631308369336935D+36
  t_f( 33)=   0.8683317618811886D+37
  t_f( 34)=   0.2952327990396041D+39
  t_f( 35)=   0.1033314796638614D+41
  t_f( 36)=   0.3719933267899012D+42
  t_f( 37)=   0.1376375309122634D+44
  t_f( 38)=   0.5230226174666010D+45
  t_f( 39)=   0.2039788208119744D+47
  t_f( 40)=   0.8159152832478977D+48
  t_f( 41)=   0.3345252661316380D+50
  t_f( 42)=   0.1405006117752880D+52
  t_f( 43)=   0.6041526306337383D+53
  t_f( 44)=   0.2658271574788449D+55
  t_f( 45)=   0.1196222208654802D+57
  t_f( 46)=   0.5502622159812088D+58
  t_f( 47)=   0.2586232415111682D+60
  t_f( 48)=   0.1241391559253607D+62
  t_f( 49)=   0.6082818640342675D+63
  t_f( 50)=   0.3041409320171338D+65
  t_f( 51)=   0.1551118753287382D+67
  t_f( 52)=   0.8065817517094388D+68
  t_f( 53)=   0.4274883284060025D+70
  t_f( 54)=   0.2308436973392414D+72
  t_f( 55)=   0.1269640335365828D+74
  t_f( 56)=   0.7109985878048635D+75
  t_f( 57)=   0.4052691950487722D+77
  t_f( 58)=   0.2350561331282879D+79
  t_f( 59)=   0.1386831185456899D+81
  t_f( 60)=   0.8320987112741392D+82
  t_f( 61)=   0.5075802138772248D+84
  t_f( 62)=   0.3146997326038794D+86
  t_f( 63)=   0.1982608315404440D+88
  t_f( 64)=   0.1268869321858842D+90
  t_f( 65)=   0.8247650592082472D+91
  t_f( 66)=   0.5443449390774431D+93
  t_f( 67)=   0.3647111091818868D+95
  t_f( 68)=   0.2480035542436831D+97
  t_f( 69)=   0.1711224524281413D+99
  t_f( 70)=  0.1197857166996989D+101
  t_f( 71)=  0.8504785885678622D+102
  t_f( 72)=  0.6123445837688608D+104
  t_f( 73)=  0.4470115461512683D+106
  t_f( 74)=  0.3307885441519386D+108
  t_f( 75)=  0.2480914081139539D+110
  t_f( 76)=  0.1885494701666050D+112
  t_f( 77)=  0.1451830920282858D+114
  t_f( 78)=  0.1132428117820629D+116
  t_f( 79)=  0.8946182130782973D+117
  t_f( 80)=  0.7156945704626378D+119
  t_f( 81)=  0.5797126020747366D+121
  t_f( 82)=  0.4753643337012840D+123
  t_f( 83)=  0.3945523969720657D+125
  t_f( 84)=  0.3314240134565352D+127
  t_f( 85)=  0.2817104114380549D+129
  t_f( 86)=  0.2422709538367272D+131
  t_f( 87)=  0.2107757298379527D+133
  t_f( 88)=  0.1854826422573984D+135
  t_f( 89)=  0.1650795516090845D+137
  t_f( 90)=  0.1485715964481761D+139
  t_f( 91)=  0.1352001527678402D+141
  t_f( 92)=  0.1243841405464130D+143
  t_f( 93)=  0.1156772507081641D+145
  t_f( 94)=  0.1087366156656742D+147
  t_f( 95)=  0.1032997848823905D+149
  t_f( 96)=  0.9916779348709491D+150
  t_f( 97)=  0.9619275968248206D+152
  t_f( 98)=  0.9426890448883242D+154
  t_f( 99)=  0.9332621544394410D+156
  t_f(100)=  0.9332621544394410D+158
  t_f(101)=  0.9425947759838354D+160
  t_f(102)=  0.9614466715035121D+162
  t_f(103)=  0.9902900716486175D+164
  t_f(104)=  0.1029901674514562D+167
  t_f(105)=  0.1081396758240290D+169
  t_f(106)=  0.1146280563734708D+171
  t_f(107)=  0.1226520203196137D+173
  t_f(108)=  0.1324641819451828D+175
  t_f(109)=  0.1443859583202493D+177
  t_f(110)=  0.1588245541522742D+179
  t_f(111)=  0.1762952551090244D+181
  t_f(112)=  0.1974506857221073D+183
  t_f(113)=  0.2231192748659812D+185
  t_f(114)=  0.2543559733472186D+187
  t_f(115)=  0.2925093693493014D+189
  t_f(116)=  0.3393108684451897D+191
  t_f(117)=  0.3969937160808719D+193
  t_f(118)=  0.4684525849754288D+195
  t_f(119)=  0.5574585761207603D+197
  t_f(120)=  0.6689502913449124D+199
  t_f(121)=  0.8094298525273440D+201
  t_f(122)=  0.9875044200833598D+203
  t_f(123)=  0.1214630436702532D+206
  t_f(124)=  0.1506141741511140D+208
  t_f(125)=  0.1882677176888925D+210
  t_f(126)=  0.2372173242880046D+212
  t_f(127)=  0.3012660018457658D+214
  t_f(128)=  0.3856204823625803D+216
  t_f(129)=  0.4974504222477285D+218
  t_f(130)=  0.6466855489220472D+220
  t_f(131)=  0.8471580690878817D+222
  t_f(132)=  0.1118248651196004D+225
  t_f(133)=  0.1487270706090685D+227
  t_f(134)=  0.1992942746161518D+229
  t_f(135)=  0.2690472707318050D+231
  t_f(136)=  0.3659042881952547D+233
  t_f(137)=  0.5012888748274990D+235
  t_f(138)=  0.6917786472619486D+237
  t_f(139)=  0.9615723196941086D+239
  t_f(140)=  0.1346201247571752D+242
  t_f(141)=  0.1898143759076170D+244
  t_f(142)=  0.2695364137888161D+246
  t_f(143)=  0.3854370717180071D+248
  t_f(144)=  0.5550293832739301D+250
  t_f(145)=  0.8047926057471987D+252
  t_f(146)=  0.1174997204390910D+255
  t_f(147)=  0.1727245890454638D+257
  t_f(148)=  0.2556323917872864D+259
  t_f(149)=  0.3808922637630567D+261
  t_f(150)=  0.5713383956445850D+263
  t_f(151)=  0.8627209774233235D+265
  t_f(152)=  0.1311335885683452D+268
  t_f(153)=  0.2006343905095681D+270
  t_f(154)=  0.3089769613847349D+272
  t_f(155)=  0.4789142901463391D+274
  t_f(156)=  0.7471062926282891D+276
  t_f(157)=  0.1172956879426414D+279
  t_f(158)=  0.1853271869493734D+281
  t_f(159)=  0.2946702272495037D+283
  t_f(160)=  0.4714723635992059D+285
  t_f(161)=  0.7590705053947215D+287
  t_f(162)=  0.1229694218739449D+290
  t_f(163)=  0.2004401576545302D+292
  t_f(164)=  0.3287218585534294D+294
  t_f(165)=  0.5423910666131586D+296
  t_f(166)=  0.9003691705778433D+298
  t_f(167)=  0.1503616514864998D+301
  t_f(168)=  0.2526075744973197D+303
  t_f(169)=  0.4269068009004703D+305
  t_f(170)=  0.7257415615307994D+307
ENDIF

fact = t_f(n)

RETURN
END FUNCTION fact

!***********************************************************************

FUNCTION lfact(n)
USE jm_constants

! Logarithm to the factorial.

IMPLICIT none

INTEGER, INTENT(IN) :: n
REAL(KIND=dp)       :: lfact
REAL(KIND=dp), SAVE :: t_lf(0:500)


IF (n < 0)   STOP '(lfact) Error: negative argument.'
IF (n > 500) STOP '(lfact) Error: argument too big.'

IF (t_lf(9) < ten) THEN               ! First call
 t_lf( 0)=  0.000000000000000D0
 t_lf( 1)=  0.000000000000000D0
 t_lf( 2)=  0.693147180559945D0
 t_lf( 3)=   1.79175946922805D0
 t_lf( 4)=   3.17805383034794D0
 t_lf( 5)=   4.78749174278204D0
 t_lf( 6)=   6.57925121201010D0
 t_lf( 7)=   8.52516136106541D0
 t_lf( 8)=   10.6046029027453D0
 t_lf( 9)=   12.8018274800815D0
 t_lf(10)=   15.1044125730755D0
 t_lf(11)=   17.5023078458739D0
 t_lf(12)=   19.9872144956619D0
 t_lf(13)=   22.5521638531234D0
 t_lf(14)=   25.1912211827387D0
 t_lf(15)=   27.8992713838409D0
 t_lf(16)=   30.6718601060807D0
 t_lf(17)=   33.5050734501369D0
 t_lf(18)=   36.3954452080331D0
 t_lf(19)=   39.3398841871995D0
 t_lf(20)=   42.3356164607535D0
 t_lf(21)=   45.3801388984769D0
 t_lf(22)=   48.4711813518352D0
 t_lf(23)=   51.6066755677644D0
 t_lf(24)=   54.7847293981123D0
 t_lf(25)=   58.0036052229805D0
 t_lf(26)=   61.2617017610020D0
 t_lf(27)=   64.5575386270063D0
 t_lf(28)=   67.8897431371815D0
 t_lf(29)=   71.2570389671680D0
 t_lf(30)=   74.6582363488301D0
 t_lf(31)=   78.0922235533153D0
 t_lf(32)=   81.5579594561150D0
 t_lf(33)=   85.0544670175815D0
 t_lf(34)=   88.5808275421977D0
 t_lf(35)=   92.1361756036871D0
 t_lf(36)=   95.7196945421432D0
 t_lf(37)=   99.3306124547874D0
 t_lf(38)=   102.968198614514D0
 t_lf(39)=   106.631760260643D0
 t_lf(40)=   110.320639714757D0
 t_lf(41)=   114.034211781462D0
 t_lf(42)=   117.771881399745D0
 t_lf(43)=   121.533081515439D0
 t_lf(44)=   125.317271149357D0
 t_lf(45)=   129.123933639127D0
 t_lf(46)=   132.952575035616D0
 t_lf(47)=   136.802722637326D0
 t_lf(48)=   140.673923648234D0
 t_lf(49)=   144.565743946345D0
 t_lf(50)=   148.477766951773D0
 t_lf(51)=   152.409592584497D0
 t_lf(52)=   156.360836303079D0
 t_lf(53)=   160.331128216631D0
 t_lf(54)=   164.320112263195D0
 t_lf(55)=   168.327445448428D0
 t_lf(56)=   172.352797139163D0
 t_lf(57)=   176.395848406997D0
 t_lf(58)=   180.456291417544D0
 t_lf(59)=   184.533828861449D0
 t_lf(60)=   188.628173423672D0
 t_lf(61)=   192.739047287845D0
 t_lf(62)=   196.866181672890D0
 t_lf(63)=   201.009316399281D0
 t_lf(64)=   205.168199482641D0
 t_lf(65)=   209.342586752537D0
 t_lf(66)=   213.532241494563D0
 t_lf(67)=   217.736934113954D0
 t_lf(68)=   221.956441819130D0
 t_lf(69)=   226.190548323728D0
 t_lf(70)=   230.439043565777D0
 t_lf(71)=   234.701723442818D0
 t_lf(72)=   238.978389561834D0
 t_lf(73)=   243.268849002983D0
 t_lf(74)=   247.572914096187D0
 t_lf(75)=   251.890402209723D0
 t_lf(76)=   256.221135550009D0
 t_lf(77)=   260.564940971863D0
 t_lf(78)=   264.921649798553D0
 t_lf(79)=   269.291097651020D0
 t_lf(80)=   273.673124285694D0
 t_lf(81)=   278.067573440366D0
 t_lf(82)=   282.474292687630D0
 t_lf(83)=   286.893133295427D0
 t_lf(84)=   291.323950094270D0
 t_lf(85)=   295.766601350761D0
 t_lf(86)=   300.220948647014D0
 t_lf(87)=   304.686856765669D0
 t_lf(88)=   309.164193580147D0
 t_lf(89)=   313.652829949879D0
 t_lf(90)=   318.152639620209D0
 t_lf(91)=   322.663499126726D0
 t_lf(92)=   327.185287703775D0
 t_lf(93)=   331.717887196928D0
 t_lf(94)=   336.261181979198D0
 t_lf(95)=   340.815058870799D0
 t_lf(96)=   345.379407062267D0
 t_lf(97)=   349.954118040770D0
 t_lf(98)=   354.539085519441D0
 t_lf(99)=   359.134205369575D0
 t_lf(100)=   363.739375555563D0
 t_lf(101)=   368.354496072405D0
 t_lf(102)=   372.979468885689D0
 t_lf(103)=   377.614197873919D0
 t_lf(104)=   382.258588773060D0
 t_lf(105)=   386.912549123218D0
 t_lf(106)=   391.575988217330D0
 t_lf(107)=   396.248817051791D0
 t_lf(108)=   400.930948278916D0
 t_lf(109)=   405.622296161145D0
 t_lf(110)=   410.322776526937D0
 t_lf(111)=   415.032306728250D0
 t_lf(112)=   419.750805599545D0
 t_lf(113)=   424.478193418257D0
 t_lf(114)=   429.214391866652D0
 t_lf(115)=   433.959323995015D0
 t_lf(116)=   438.712914186121D0
 t_lf(117)=   443.475088120919D0
 t_lf(118)=   448.245772745385D0
 t_lf(119)=   453.024896238496D0
 t_lf(120)=   457.812387981278D0
 t_lf(121)=   462.608178526875D0
 t_lf(122)=   467.412199571608D0
 t_lf(123)=   472.224383926981D0
 t_lf(124)=   477.044665492586D0
 t_lf(125)=   481.872979229888D0
 t_lf(126)=   486.709261136839D0
 t_lf(127)=   491.553448223298D0
 t_lf(128)=   496.405478487218D0
 t_lf(129)=   501.265290891579D0
 t_lf(130)=   506.132825342035D0
 t_lf(131)=   511.008022665236D0
 t_lf(132)=   515.890824587822D0
 t_lf(133)=   520.781173716044D0
 t_lf(134)=   525.679013515995D0
 t_lf(135)=   530.584288294433D0
 t_lf(136)=   535.496943180170D0
 t_lf(137)=   540.416924105998D0
 t_lf(138)=   545.344177791155D0
 t_lf(139)=   550.278651724286D0
 t_lf(140)=   555.220294146895D0
 t_lf(141)=   560.169054037273D0
 t_lf(142)=   565.124881094874D0
 t_lf(143)=   570.087725725134D0
 t_lf(144)=   575.057539024710D0
 t_lf(145)=   580.034272767131D0
 t_lf(146)=   585.017879388839D0
 t_lf(147)=   590.008311975618D0
 t_lf(148)=   595.005524249382D0
 t_lf(149)=   600.009470555327D0
 t_lf(150)=   605.020105849424D0
 t_lf(151)=   610.037385686239D0
 t_lf(152)=   615.061266207085D0
 t_lf(153)=   620.091704128477D0
 t_lf(154)=   625.128656730891D0
 t_lf(155)=   630.172081847810D0
 t_lf(156)=   635.221937855060D0
 t_lf(157)=   640.278183660408D0
 t_lf(158)=   645.340778693435D0
 t_lf(159)=   650.409682895655D0
 t_lf(160)=   655.484856710889D0
 t_lf(161)=   660.566261075873D0
 t_lf(162)=   665.653857411106D0
 t_lf(163)=   670.747607611913D0
 t_lf(164)=   675.847474039737D0
 t_lf(165)=   680.953419513637D0
 t_lf(166)=   686.065407301994D0
 t_lf(167)=   691.183401114411D0
 t_lf(168)=   696.307365093814D0
 t_lf(169)=   701.437263808737D0
 t_lf(170)=   706.573062245787D0
 t_lf(171)=   711.714725802290D0
 t_lf(172)=   716.862220279103D0
 t_lf(173)=   722.015511873601D0
 t_lf(174)=   727.174567172816D0
 t_lf(175)=   732.339353146739D0
 t_lf(176)=   737.509837141778D0
 t_lf(177)=   742.685986874351D0
 t_lf(178)=   747.867770424643D0
 t_lf(179)=   753.055156230484D0
 t_lf(180)=   758.248113081374D0
 t_lf(181)=   763.446610112640D0
 t_lf(182)=   768.650616799717D0
 t_lf(183)=   773.860102952558D0
 t_lf(184)=   779.075038710167D0
 t_lf(185)=   784.295394535246D0
 t_lf(186)=   789.521141208959D0
 t_lf(187)=   794.752249825813D0
 t_lf(188)=   799.988691788643D0
 t_lf(189)=   805.230438803703D0
 t_lf(190)=   810.477462875864D0
 t_lf(191)=   815.729736303910D0
 t_lf(192)=   820.987231675938D0
 t_lf(193)=   826.249921864843D0
 t_lf(194)=   831.517780023906D0
 t_lf(195)=   836.790779582470D0
 t_lf(196)=   842.068894241700D0
 t_lf(197)=   847.352097970438D0
 t_lf(198)=   852.640365001133D0
 t_lf(199)=   857.933669825857D0
 t_lf(200)=   863.231987192405D0
 t_lf(201)=   868.535292100464D0
 t_lf(202)=   873.843559797866D0
 t_lf(203)=   879.156765776907D0
 t_lf(204)=   884.474885770752D0
 t_lf(205)=   889.797895749890D0
 t_lf(206)=   895.125771918680D0
 t_lf(207)=   900.458490711945D0
 t_lf(208)=   905.796028791646D0
 t_lf(209)=   911.138363043611D0
 t_lf(210)=   916.485470574329D0
 t_lf(211)=   921.837328707805D0
 t_lf(212)=   927.193914982477D0
 t_lf(213)=   932.555207148186D0
 t_lf(214)=   937.921183163208D0
 t_lf(215)=   943.291821191336D0
 t_lf(216)=   948.667099599020D0
 t_lf(217)=   954.046996952560D0
 t_lf(218)=   959.431492015349D0
 t_lf(219)=   964.820563745166D0
 t_lf(220)=   970.214191291518D0
 t_lf(221)=   975.612353993036D0
 t_lf(222)=   981.015031374908D0
 t_lf(223)=   986.422203146368D0
 t_lf(224)=   991.833849198223D0
 t_lf(225)=   997.249949600428D0
 t_lf(226)=   1002.67048459970D0
 t_lf(227)=   1008.09543461718D0
 t_lf(228)=   1013.52478024614D0
 t_lf(229)=   1018.95850224969D0
 t_lf(230)=   1024.39658155861D0
 t_lf(231)=   1029.83899926914D0
 t_lf(232)=   1035.28573664080D0
 t_lf(233)=   1040.73677509437D0
 t_lf(234)=   1046.19209620972D0
 t_lf(235)=   1051.65168172387D0
 t_lf(236)=   1057.11551352889D0
 t_lf(237)=   1062.58357367003D0
 t_lf(238)=   1068.05584434370D0
 t_lf(239)=   1073.53230789563D0
 t_lf(240)=   1079.01294681897D0
 t_lf(241)=   1084.49774375247D0
 t_lf(242)=   1089.98668147862D0
 t_lf(243)=   1095.47974292196D0
 t_lf(244)=   1100.97691114726D0
 t_lf(245)=   1106.47816935780D0
 t_lf(246)=   1111.98350089373D0
 t_lf(247)=   1117.49288923036D0
 t_lf(248)=   1123.00631797652D0
 t_lf(249)=   1128.52377087299D0
 t_lf(250)=   1134.04523179085D0
 t_lf(251)=   1139.57068472998D0
 t_lf(252)=   1145.10011381750D0
 t_lf(253)=   1150.63350330622D0
 t_lf(254)=   1156.17083757324D0
 t_lf(255)=   1161.71210111840D0
 t_lf(256)=   1167.25727856288D0
 t_lf(257)=   1172.80635464777D0
 t_lf(258)=   1178.35931423270D0
 t_lf(259)=   1183.91614229440D0
 t_lf(260)=   1189.47682392541D0
 t_lf(261)=   1195.04134433273D0
 t_lf(262)=   1200.60968883650D0
 t_lf(263)=   1206.18184286867D0
 t_lf(264)=   1211.75779197182D0
 t_lf(265)=   1217.33752179781D0
 t_lf(266)=   1222.92101810659D0
 t_lf(267)=   1228.50826676499D0
 t_lf(268)=   1234.09925374550D0
 t_lf(269)=   1239.69396512510D0
 t_lf(270)=   1245.29238708410D0
 t_lf(271)=   1250.89450590498D0
 t_lf(272)=   1256.50030797127D0
 t_lf(273)=   1262.10977976646D0
 t_lf(274)=   1267.72290787285D0
 t_lf(275)=   1273.33967897051D0
 t_lf(276)=   1278.96007983623D0
 t_lf(277)=   1284.58409734242D0
 t_lf(278)=   1290.21171845611D0
 t_lf(279)=   1295.84293023793D0
 t_lf(280)=   1301.47771984110D0
 t_lf(281)=   1307.11607451043D0
 t_lf(282)=   1312.75798158137D0
 t_lf(283)=   1318.40342847902D0
 t_lf(284)=   1324.05240271718D0
 t_lf(285)=   1329.70489189744D0
 t_lf(286)=   1335.36088370826D0
 t_lf(287)=   1341.02036592402D0
 t_lf(288)=   1346.68332640416D0
 t_lf(289)=   1352.34975309227D0
 t_lf(290)=   1358.01963401525D0
 t_lf(291)=   1363.69295728242D0
 t_lf(292)=   1369.36971108469D0
 t_lf(293)=   1375.04988369371D0
 t_lf(294)=   1380.73346346105D0
 t_lf(295)=   1386.42043881739D0
 t_lf(296)=   1392.11079827171D0
 t_lf(297)=   1397.80453041052D0
 t_lf(298)=   1403.50162389702D0
 t_lf(299)=   1409.20206747041D0
 t_lf(300)=   1414.90584994507D0
 t_lf(301)=   1420.61296020982D0
 t_lf(302)=   1426.32338722719D0
 t_lf(303)=   1432.03712003270D0
 t_lf(304)=   1437.75414773411D0
 t_lf(305)=   1443.47445951071D0
 t_lf(306)=   1449.19804461267D0
 t_lf(307)=   1454.92489236025D0
 t_lf(308)=   1460.65499214323D0
 t_lf(309)=   1466.38833342013D0
 t_lf(310)=   1472.12490571760D0
 t_lf(311)=   1477.86469862978D0
 t_lf(312)=   1483.60770181759D0
 t_lf(313)=   1489.35390500813D0
 t_lf(314)=   1495.10329799404D0
 t_lf(315)=   1500.85587063287D0
 t_lf(316)=   1506.61161284645D0
 t_lf(317)=   1512.37051462033D0
 t_lf(318)=   1518.13256600311D0
 t_lf(319)=   1523.89775710590D0
 t_lf(320)=   1529.66607810169D0
 t_lf(321)=   1535.43751922482D0
 t_lf(322)=   1541.21207077036D0
 t_lf(323)=   1546.98972309359D0
 t_lf(324)=   1552.77046660938D0
 t_lf(325)=   1558.55429179171D0
 t_lf(326)=   1564.34118917308D0
 t_lf(327)=   1570.13114934397D0
 t_lf(328)=   1575.92416295236D0
 t_lf(329)=   1581.72022070312D0
 t_lf(330)=   1587.51931335758D0
 t_lf(331)=   1593.32143173296D0
 t_lf(332)=   1599.12656670188D0
 t_lf(333)=   1604.93470919186D0
 t_lf(334)=   1610.74585018483D0
 t_lf(335)=   1616.55998071666D0
 t_lf(336)=   1622.37709187662D0
 t_lf(337)=   1628.19717480697D0
 t_lf(338)=   1634.02022070246D0
 t_lf(339)=   1639.84622080984D0
 t_lf(340)=   1645.67516642745D0
 t_lf(341)=   1651.50704890473D0
 t_lf(342)=   1657.34185964179D0
 t_lf(343)=   1663.17959008896D0
 t_lf(344)=   1669.02023174633D0
 t_lf(345)=   1674.86377616336D0
 t_lf(346)=   1680.71021493842D0
 t_lf(347)=   1686.55953971837D0
 t_lf(348)=   1692.41174219814D0
 t_lf(349)=   1698.26681412035D0
 t_lf(350)=   1704.12474727483D0
 t_lf(351)=   1709.98553349830D0
 t_lf(352)=   1715.84916467389D0
 t_lf(353)=   1721.71563273083D0
 t_lf(354)=   1727.58492964396D0
 t_lf(355)=   1733.45704743344D0
 t_lf(356)=   1739.33197816429D0
 t_lf(357)=   1745.20971394607D0
 t_lf(358)=   1751.09024693247D0
 t_lf(359)=   1756.97356932096D0
 t_lf(360)=   1762.85967335241D0
 t_lf(361)=   1768.74855131074D0
 t_lf(362)=   1774.64019552257D0
 t_lf(363)=   1780.53459835683D0
 t_lf(364)=   1786.43175222447D0
 t_lf(365)=   1792.33164957805D0
 t_lf(366)=   1798.23428291145D0
 t_lf(367)=   1804.13964475951D0
 t_lf(368)=   1810.04772769768D0
 t_lf(369)=   1815.95852434172D0
 t_lf(370)=   1821.87202734735D0
 t_lf(371)=   1827.78822940996D0
 t_lf(372)=   1833.70712326423D0
 t_lf(373)=   1839.62870168388D0
 t_lf(374)=   1845.55295748129D0
 t_lf(375)=   1851.47988350726D0
 t_lf(376)=   1857.40947265065D0
 t_lf(377)=   1863.34171783810D0
 t_lf(378)=   1869.27661203372D0
 t_lf(379)=   1875.21414823880D0
 t_lf(380)=   1881.15431949152D0
 t_lf(381)=   1887.09711886665D0
 t_lf(382)=   1893.04253947526D0
 t_lf(383)=   1898.99057446444D0
 t_lf(384)=   1904.94121701703D0
 t_lf(385)=   1910.89446035131D0
 t_lf(386)=   1916.85029772078D0
 t_lf(387)=   1922.80872241381D0
 t_lf(388)=   1928.76972775343D0
 t_lf(389)=   1934.73330709705D0
 t_lf(390)=   1940.69945383617D0
 t_lf(391)=   1946.66816139616D0
 t_lf(392)=   1952.63942323595D0
 t_lf(393)=   1958.61323284782D0
 t_lf(394)=   1964.58958375712D0
 t_lf(395)=   1970.56846952202D0
 t_lf(396)=   1976.54988373327D0
 t_lf(397)=   1982.53382001396D0
 t_lf(398)=   1988.52027201924D0
 t_lf(399)=   1994.50923343613D0
 t_lf(400)=   2000.50069798324D0
 t_lf(401)=   2006.49465941055D0
 t_lf(402)=   2012.49111149917D0
 t_lf(403)=   2018.49004806111D0
 t_lf(404)=   2024.49146293907D0
 t_lf(405)=   2030.49535000618D0
 t_lf(406)=   2036.50170316578D0
 t_lf(407)=   2042.51051635123D0
 t_lf(408)=   2048.52178352563D0
 t_lf(409)=   2054.53549868167D0
 t_lf(410)=   2060.55165584137D0
 t_lf(411)=   2066.57024905587D0
 t_lf(412)=   2072.59127240522D0
 t_lf(413)=   2078.61471999818D0
 t_lf(414)=   2084.64058597200D0
 t_lf(415)=   2090.66886449223D0
 t_lf(416)=   2096.69954975249D0
 t_lf(417)=   2102.73263597429D0
 t_lf(418)=   2108.76811740682D0
 t_lf(419)=   2114.80598832674D0
 t_lf(420)=   2120.84624303802D0
 t_lf(421)=   2126.88887587170D0
 t_lf(422)=   2132.93388118574D0
 t_lf(423)=   2138.98125336478D0
 t_lf(424)=   2145.03098682001D0
 t_lf(425)=   2151.08307598894D0
 t_lf(426)=   2157.13751533521D0
 t_lf(427)=   2163.19429934844D0
 t_lf(428)=   2169.25342254402D0
 t_lf(429)=   2175.31487946295D0
 t_lf(430)=   2181.37866467163D0
 t_lf(431)=   2187.44477276174D0
 t_lf(432)=   2193.51319834998D0
 t_lf(433)=   2199.58393607798D0
 t_lf(434)=   2205.65698061209D0
 t_lf(435)=   2211.73232664317D0
 t_lf(436)=   2217.80996888652D0
 t_lf(437)=   2223.88990208162D0
 t_lf(438)=   2229.97212099199D0
 t_lf(439)=   2236.05662040507D0
 t_lf(440)=   2242.14339513198D0
 t_lf(441)=   2248.23244000743D0
 t_lf(442)=   2254.32374988951D0
 t_lf(443)=   2260.41731965955D0
 t_lf(444)=   2266.51314422198D0
 t_lf(445)=   2272.61121850415D0
 t_lf(446)=   2278.71153745617D0
 t_lf(447)=   2284.81409605078D0
 t_lf(448)=   2290.91888928320D0
 t_lf(449)=   2297.02591217094D0
 t_lf(450)=   2303.13515975371D0
 t_lf(451)=   2309.24662709321D0
 t_lf(452)=   2315.36030927304D0
 t_lf(453)=   2321.47620139852D0
 t_lf(454)=   2327.59429859656D0
 t_lf(455)=   2333.71459601552D0
 t_lf(456)=   2339.83708882503D0
 t_lf(457)=   2345.96177221592D0
 t_lf(458)=   2352.08864140004D0
 t_lf(459)=   2358.21769161010D0
 t_lf(460)=   2364.34891809958D0
 t_lf(461)=   2370.48231614258D0
 t_lf(462)=   2376.61788103366D0
 t_lf(463)=   2382.75560808775D0
 t_lf(464)=   2388.89549263997D0
 t_lf(465)=   2395.03753004556D0
 t_lf(466)=   2401.18171567969D0
 t_lf(467)=   2407.32804493736D0
 t_lf(468)=   2413.47651323327D0
 t_lf(469)=   2419.62711600172D0
 t_lf(470)=   2425.77984869642D0
 t_lf(471)=   2431.93470679044D0
 t_lf(472)=   2438.09168577602D0
 t_lf(473)=   2444.25078116452D0
 t_lf(474)=   2450.41198848621D0
 t_lf(475)=   2456.57530329025D0
 t_lf(476)=   2462.74072114448D0
 t_lf(477)=   2468.90823763537D0
 t_lf(478)=   2475.07784836786D0
 t_lf(479)=   2481.24954896527D0
 t_lf(480)=   2487.42333506917D0
 t_lf(481)=   2493.59920233928D0
 t_lf(482)=   2499.77714645333D0
 t_lf(483)=   2505.95716310698D0
 t_lf(484)=   2512.13924801370D0
 t_lf(485)=   2518.32339690463D0
 t_lf(486)=   2524.50960552853D0
 t_lf(487)=   2530.69786965162D0
 t_lf(488)=   2536.88818505747D0
 t_lf(489)=   2543.08054754695D0
 t_lf(490)=   2549.27495293805D0
 t_lf(491)=   2555.47139706584D0
 t_lf(492)=   2561.66987578234D0
 t_lf(493)=   2567.87038495638D0
 t_lf(494)=   2574.07292047357D0
 t_lf(495)=   2580.27747823614D0
 t_lf(496)=   2586.48405416286D0
 t_lf(497)=   2592.69264418896D0
 t_lf(498)=   2598.90324426598D0
 t_lf(499)=   2605.11585036173D0
 t_lf(500)=   2611.33045846016D0
ENDIF

lfact = t_lf(n)

RETURN
END FUNCTION lfact

!***********************************************************************

FUNCTION hyperg11(a,c,z)
USE jm_constants

! The hypergeometric function 1F1(a,c;z)

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: a, c, z
REAL(KIND=dp)             :: hyperg11
REAL(KIND=dp)             :: s, d, y
INTEGER                   :: n

s = one
y = one

DO n=1,500
  d = n * ( (c+n-one)**2 )
  y = (a+n-one) * (y/d)
  y = y * (c+n-one)
  y = y * z
  IF (s == s+y) EXIT
  s = s + y
ENDDO

IF (n == 500) STOP '(hyperg11) Error: convergence fail.'

hyperg11 = s

RETURN
END FUNCTION hyperg11

!***********************************************************************

FUNCTION hyperg21(a,b,c,z)
USE jm_constants

! The hypergeometric function 2F1(a,b,c;z)

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: a, b, c, z
REAL(KIND=dp)             :: hyperg21
REAL(KIND=dp)             :: s, y
INTEGER                   :: n

s = one
y = one

DO n=0,500
  y = y*(a+n)/(c+n)*(b+n)*z/(n+1)
  IF (s == s+y ) EXIT
  s = s+y
ENDDO

IF (n == 500) STOP '(hyperg21) Error: convergence fail.'

hyperg21 = s

RETURN
END FUNCTION hyperg21

!***********************************************************************

FUNCTION hyperg01(c,z)
USE jm_constants

! The hypergeometric function 0F1(c;z)

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: c, z
REAL(KIND=dp)             :: hyperg01
REAL(KIND=dp)             :: s, d, y
INTEGER                   :: n

s = one
y = one

DO n=1,500
  d = n * ( (c+n-one)**2 )
  y = y/d
  y = y * (c+n-one)
  y = y * z
  IF (s == s+y) EXIT
  s = s + y
ENDDO

IF (n == 500) STOP '(hyperg01): Error: convergence fail.'

hyperg01 = s

RETURN
END FUNCTION hyperg01

!***********************************************************************