FUNCTION v(r)
USE jm_constants
USE jm_global, ONLY: a, b, v0, r0, z, alpha, pot_type

! Formulas for the scattering potential.

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: r
REAL(KIND=dp)             :: v


IF (pot_type == 'well') THEN    ! Potential square-well

  IF ( r >= a .AND. r <= b ) THEN
    v =  v0
  ELSE
    v = zero
  ENDIF
  
ELSE IF (pot_type == 'coulomb') THEN  

  IF (r < r0) THEN
    v = - ( z / r**alpha )
  ELSE
    v = zero
  ENDIF
  
ELSE    ! The other potential

! Put your formula here
  v = - 30.0_dp / r
  
ENDIF

RETURN
END FUNCTION v

!***********************************************************************

FUNCTION vn(i,j)
USE jm_constants
USE jm_global, ONLY: pot_type, scheme, ntrunc, a, b, nstart, nend, pass, r0, vn_is_loaded

! Truncates the scattering potential in the selected basis.

IMPLICIT none

INTEGER, INTENT(IN)     :: i, j
REAL(KIND=dp)           :: vn
INTEGER                 :: ii, jj, dim
REAL(KIND=dp)           :: integral_inf, integral
REAL(KIND=dp), EXTERNAL :: fi_v_fi, psi_v_psi
REAL(KIND=dp), DIMENSION(:,:), ALLOCATABLE, SAVE :: v

COMMON /mn/ii,jj 


IF ((.NOT. vn_is_loaded) .AND.  &
     (pass == 2 .OR. (pass ==1 .AND. nstart == ntrunc))) THEN
  dim = ntrunc
  IF (scheme == 'relativistic') dim = 2 * ntrunc
  ALLOCATE( v(0:dim-1,0:dim-1) )
  CALL read_vn_from_file(dim,v,vn_is_loaded)
  IF (.NOT. vn_is_loaded) DEALLOCATE(v)
ENDIF

IF (vn_is_loaded) THEN

  vn = v(i,j)

!open(222,file="test")
!      DO ii=1,dim
!        DO jj=1,dim
!           WRITE(222,*) v(ii-1,jj-1)
!        ENDDO            
!      ENDDO
!close(222)
!stop  
ELSE

  IF (scheme == 'relativistic') THEN
  
    IF ( (i < ntrunc) .AND. (j < ntrunc) ) THEN   ! The "++" area

      ii = ntrunc-i-1
      jj = ntrunc-j-1

      IF (pot_type == 'well') THEN
        vn = integral(fi_v_fi, a, b)
      ELSE IF (pot_type == 'coulomb') THEN
        vn = integral(fi_v_fi, zero, r0)
      ELSE
        vn = integral_inf(fi_v_fi, zero)
      ENDIF
    ELSE IF ((i >= ntrunc) .AND. (j >= ntrunc) ) THEN  ! The "--" area

      ii = i-ntrunc
      jj = j-ntrunc
    
      IF (pot_type == 'well') THEN
        vn = integral(psi_v_psi, a, b)
      ELSE IF (pot_type == 'coulomb') THEN
        vn = integral(fi_v_fi, zero, r0)
      ELSE
        vn = integral_inf(psi_v_psi, zero)
      ENDIF
    
    ELSE    ! The "+-" and "-+" areas

      vn = zero
    
    ENDIF
  
  ELSE  ! Non-relativistic formulas

    ii = i
    jj = j
  
    IF ( (i < ntrunc) .AND. (j < ntrunc) ) THEN
    
      IF (pot_type == 'well') THEN         ! Square-well -
        vn = integral(fi_v_fi, a, b)       ! - integrate from a to b
      ELSE IF (pot_type == 'coulomb') THEN ! Truncated coulomb potential -
        vn = integral(fi_v_fi, zero, r0)   ! - integrate from 0 to r0
      ELSE
        vn = integral_inf(fi_v_fi, zero)   ! Integrate from 0 to infinity
      ENDIF
    
    ELSE
      vn = zero
    ENDIF
  
  ENDIF

ENDIF

RETURN


END FUNCTION vn

!***********************************************************************

FUNCTION fi_v_fi(r)
USE jm_constants

! Forms function to integrate.

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: r
REAL(KIND=dp)             :: fi_v_fi
REAL(KIND=dp)             :: v, fi
INTEGER                   :: m, n

COMMON /mn/m,n

fi_v_fi = fi(m,r) * v(r) * fi(n,r)

RETURN
END FUNCTION fi_v_fi

!***********************************************************************

FUNCTION psi_v_psi(r)
USE jm_constants

! Forms function to integrate.

IMPLICIT none

REAL(KIND=dp), INTENT(IN) :: r
REAL(KIND=dp)             :: psi_v_psi
REAL(KIND=dp)             :: v, psi
INTEGER                   :: m, n

COMMON /mn/m,n

psi_v_psi = psi(m,r) * v(r) * psi(n,r)

RETURN
END FUNCTION psi_v_psi

!***********************************************************************


SUBROUTINE read_vn_from_file(dim,v,vn_is_loaded)
USE jm_constants
USE jm_global, ONLY: ntrunc, l, kappa, lambda, basis, scheme, scr, pass, fout

! Reads previously saved elements of the truncated potential.

IMPLICIT NONE

INTEGER,       INTENT(in)  :: dim
LOGICAL,       INTENT(out) :: vn_is_loaded      
REAL(KIND=dp), INTENT(out) :: v(0:dim-1,0:dim-1)
INTEGER                    :: nend_,l_,kappa_,i
REAL(KIND=dp)              :: lambda_
CHARACTER(LEN=8)           :: basis_
CHARACTER(LEN=16)          :: scheme_
CHARACTER(LEN=80)          :: tmp

vn_is_loaded = .FALSE.
  
OPEN(5,file='saved.vn',status='old',err=1)

DO
  READ(5,'(A80)') tmp                    ! Read and ignore the header 
  IF (index(tmp,";")==0) EXIT            ! from the input file
ENDDO
BACKSPACE(5)

! Testing if the <saved.vn> file is correct
READ(5,'(3I3,F5.2,A9,A1,A16)',err=1,end=1)   &
           nend_,l_,kappa_,lambda_,basis_,tmp,scheme_

IF (ntrunc==nend_ .AND. l==l_ .AND. kappa==kappa_ .AND. scheme==scheme_  &
    .AND. lambda==lambda_ .AND. basis==basis_) THEN
  READ(5,*,end=1) v
  vn_is_loaded = .TRUE.
  IF(scr .AND. pass == 1) WRITE(*,'(A)')  &
  '(using previously calculated and saved elements of the truncated potential)'
ENDIF

1 CLOSE(5)
RETURN

END SUBROUTINE read_vn_from_file
